package com.tigrek.iuea.ai;
import java.util.ArrayList;
import java.util.Random;
//Population class
public class Population {
	int popSize = 10;
	boolean debug=true;
	public final static double CROSSLIMIT=0.25;
	public final static double MUTATIONLIMIT=0.10;

	ArrayList<Chromosome> chromosomes;
	ArrayList<Chromosome> newchromosomes;
	Chromosome fittestChromosome,unfittestChromosome;
	double [] C;
	double [] rSelection;
	double fittest = 0;
	double total=0.0;
	Random rn = new Random();
	ArrayList<Chromosome> selection;
	ArrayList<Chromosome> crossed;
	public Population(int size) {
		popSize=size;
		chromosomes= new ArrayList<Chromosome>();
		newchromosomes= new ArrayList<Chromosome>();

		selection=new ArrayList<Chromosome>();
		crossed=new ArrayList<Chromosome>();

		C=new double[popSize];
		rSelection=new double[popSize];
		initializePopulation(popSize);
	}
	//Initialize population
	public void initializePopulation(int size) {

		for (int i = 0; i < popSize; i++) {
			Chromosome c = new Chromosome(i);
			chromosomes.add(c);
		}
	}
	public void printPopulation()
	{
		if(GA.FINAL)
			return;
		System.out.print("Population:\t");
		for (int i = 0; i < chromosomes.size(); i++) {
			chromosomes.get(i).out();
		}
		System.out.println();
	}
	//Calculate fitness of each individual
	public void calculateFitness() {
		fittest=0;
		total=0;
		for (int i = 0; i < chromosomes.size(); i++) {
			chromosomes.get(i).calcFitness();
			double fit=chromosomes.get(i).fitness;
			total +=fit;
			if(fit>fittest)
			{
				fittest=fit;
			}
		}
	}
	public void printFitness()
	{
		if(GA.FINAL)
			return;
		System.out.print("\nFobj:\t\t");
		for (int i = 0; i < chromosomes.size(); i++) {
			System.out.printf("%d\t\t",chromosomes.get(i).fobj);
		}
		System.out.println();
		System.out.print("Fitness:\t");
		for (int i = 0; i < chromosomes.size(); i++) {
			System.out.printf("%f\t",chromosomes.get(i).fitness);
		}
		System.out.println();
		if(!GA.FINAL)
		{
			System.out.printf("Total=%f \t",total);
			System.out.printf("Fittest=%f\n",fittest);
		}
	}
	public void calculateProbability()
	{
		if(!GA.FINAL)
			System.out.print("\nProbability:\t");
		for (int i = 0; i < chromosomes.size(); i++) {
			chromosomes.get(i).calcProb(total);
			if(!GA.FINAL)
				System.out.printf("%f\t",chromosomes.get(i).probability);
		}
		if(!GA.FINAL)
			System.out.println();
	}
	public void calculateCumulativeProb()
	{
		if(!GA.FINAL)
			System.out.print("Cum.Prob:\t");
		C[0]=chromosomes.get(0).probability;
		if(!GA.FINAL)
			System.out.printf("%f\t",C[0]);
		for (int i = 1; i < chromosomes.size(); i++) {
			C[i]=C[i-1]+chromosomes.get(i).probability;
			if(!GA.FINAL)
				System.out.printf("%f\t",C[i]);
		}
		if(!GA.FINAL)
			System.out.println();
	}
	public void Selection()
	{
		newchromosomes= new ArrayList<Chromosome>();
		for (int i = 0; i < chromosomes.size(); i++) {
			double r=Math.random();
			rSelection[i]=r;
			int j=0;
			boolean found=false;
			while(!found && j<chromosomes.size())
			{
				if(r<C[j])
				{
					Chromosome cc=chromosomes.get(j);
					Chromosome nc=cc.clone(j,i);
					newchromosomes.add(nc);
					found=true;
				}
				j++;
			}
		}
		printrSelection();
		printNewPopulation();
	}
	public Chromosome copyChromosome(int i)
	{
		Chromosome cc=new Chromosome(i);
		cc=newchromosomes.get(i).clone(i, i);
		return cc;
	}
	public void printrSelection()
	{
		if(GA.FINAL)
			return;
		System.out.print("rSelection:\t");
		for (int i = 0; i < rSelection.length; i++) {
			System.out.printf("%f\t",rSelection[i]);
		}
		System.out.println();
	}
	public void printNewPopulation()
	{
		if(GA.FINAL)
			return;
		System.out.print("Selection:\t");
		for (int i = 0; i < newchromosomes.size(); i++) {
			newchromosomes.get(i).out();
		}
		System.out.println();
	}
	public void CrossSelection()
	{
		selection=new ArrayList<Chromosome>();

		for (int i = 1; i < newchromosomes.size(); i++) 
		{
			double r=Math.random();
			if(r<CROSSLIMIT)
			{
				selection.add(copyChromosome(i));
			}
		}
		if(!GA.FINAL)
			System.out.print("CrossOver Number:<"+selection.size()+">\n");
	}

	public void CrossAction()
	{
		Chromosome c1,c2,c3;
		crossed=new ArrayList<Chromosome>();
		int n=selection.size();
		for(int i=0;i<n;i++)
		{
			if(!GA.FINAL)
				System.out.print("\tAction <"+(i+1)+">:");
			c1=selection.get(i);
			if(!GA.FINAL) {
				System.out.print("\t"+c1.index+"-->");
				c1.out();
			}
			if(i<n-1)
				c2=selection.get(i+1);
			else
				c2=selection.get(0);
			if(!GA.FINAL)
			{
				System.out.print("\t"+c2.index+"-->");
				c2.out();
			}
			c3=CrossOver(c1,c2);///  calculate the cross
			crossed.add(c3);
			if(!GA.FINAL)
			{
				System.out.print("\t"+c3.index+"->");
				c3.out();
				System.out.println();
			}
		}
		if(!GA.FINAL)
			System.out.println();
	}
	public Chromosome CrossOver(Chromosome c1,Chromosome c2)
	{
		Random r=new Random();
		Chromosome c3=new Chromosome();
		int n=c1.geneLength;
		int cut=r.nextInt(n-1);
		if(!GA.FINAL)
			System.out.print("\tcut#:<"+cut+">  ");
		for(int i=0;i<=cut;i++) {
			c3.genes[i]=c1.genes[i];
		}
		for(int i=cut+1;i<n;i++) {
			c3.genes[i]=c2.genes[i];
		}
		c3.setIndex(c1.getIndex());// take the place of the first chromosome in the process
		return c3;
	}
	public void CrossPlaced()
	{
		if(!GA.FINAL)
			System.out.println(crossed.size()+" CrossOver inserted");
		for (int i = 0; i < crossed.size(); i++) 
		{
			Chromosome ff=crossed.get(i);
			int place=ff.getIndex();
			if(!GA.FINAL)
			{
				System.out.print("\t<"+place+"> ");
				ff.out();
			}
			newchromosomes.set(place,ff);
		}
		if(!GA.FINAL)
			System.out.println();
	}
	public void mutation()
	{
		//MUTATIONLIMIT
		int n=newchromosomes.size();
		int nMut=2;//(int) (n*mut);
		int cl=newchromosomes.get(0).geneLength;
		Random r=new Random();
		for (int i = 0; i < nMut; i++) 
		{
			int p=r.nextInt(n);
			if(!GA.FINAL)
				System.out.print("Mutating chromosome #"+p);
			newchromosomes.get(p).mutate();
		}
	}
	public void setNewPopulation()
	{
		if(!GA.FINAL)
			System.out.println("New Generation after all process!");
		chromosomes= new ArrayList<Chromosome>();
		for (int i = 0; i < newchromosomes.size(); i++) 
		{
			chromosomes.add(newchromosomes.get(i).clone(-1, i));
			if(!GA.FINAL)
				chromosomes.get(i).out();
		}
	}
	//Get the fittest individual
	public Chromosome getFittest() {
		double maxFit = Integer.MIN_VALUE;
		int maxFitIndex = 0;
		for (int i = 0; i < chromosomes.size(); i++) {
			if (maxFit <= chromosomes.get(i).fitness) {
				maxFit = chromosomes.get(i).fitness;
				maxFitIndex = i;
			}
		}
		fittestChromosome=chromosomes.get(maxFitIndex);
		fittestChromosome.setIndex(maxFitIndex);
		fittest = chromosomes.get(maxFitIndex).fitness;
		return chromosomes.get(maxFitIndex);
	}
}